package com.sun.org.apache.bcel.internal.util;

import java.util.zip.ZipEntry;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.zip.ZipFile;
import java.io.File;
import java.util.StringTokenizer;
import jdk.xml.internal.SecuritySupport;
import java.util.ArrayList;
import java.io.Serializable;

public class ClassPath implements Serializable
{
    public static final ClassPath SYSTEM_CLASS_PATH;
    private ClassPath.PathEntry[] paths;
    private String class_path;
    
    public ClassPath(String class_path) {
        this.class_path = class_path;
        ArrayList vec = new ArrayList();
        StringTokenizer tok = new StringTokenizer(class_path, SecuritySupport.getSystemProperty("path.separator"));
        while (tok.hasMoreTokens()) {
            String path = tok.nextToken();
            if (!path.equals("")) {
                File file = new File(path);
                try {
                    if (SecuritySupport.isFileExists(file)) {
                        if (file.isDirectory()) {
                            vec.add(new Dir(path));
                        }
                        else {
                            vec.add(new ClassPath.Zip(new ZipFile(file)));
                        }
                    }
                    else {
                        continue;
                    }
                }
                catch (IOException e) {
                    System.err.println("CLASSPATH component " + file + ": " + e);
                }
            }
            else {
                continue;
            }
        }
        vec.toArray(this.paths = new ClassPath.PathEntry[vec.size()]);
    }
    
    @Deprecated
    public ClassPath() {
        this("");
    }
    
    @Override
    public String toString() {
        return this.class_path;
    }
    
    @Override
    public int hashCode() {
        return this.class_path.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof ClassPath && this.class_path.equals(((ClassPath)o).class_path);
    }
    
    private static final void getPathComponents(String path, ArrayList list) {
        if (path != null) {
            StringTokenizer tok = new StringTokenizer(path, File.pathSeparator);
            while (tok.hasMoreTokens()) {
                String name = tok.nextToken();
                File file = new File(name);
                if (SecuritySupport.isFileExists(file)) {
                    list.add(name);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    public static final String getClassPath() {
        String class_path;
        String boot_path;
        String ext_path;
        try {
            class_path = SecuritySupport.getSystemProperty("java.class.path");
            boot_path = SecuritySupport.getSystemProperty("sun.boot.class.path");
            ext_path = SecuritySupport.getSystemProperty("java.ext.dirs");
        }
        catch (SecurityException e) {
            return "";
        }
        ArrayList<String> list = new ArrayList();
        getPathComponents(class_path, list);
        getPathComponents(boot_path, list);
        ArrayList<Object> dirs = new ArrayList();
        getPathComponents(ext_path, dirs);
        Iterator<Object> e2 = dirs.iterator();
        while (e2.hasNext()) {
            File ext_dir = new File((String)e2.next());
            String[] extensions = SecuritySupport.getFileList(ext_dir, new FilenameFilter() {
                @Override
                public boolean accept(File dir, String name) {
                    name = name.toLowerCase();
                    return name.endsWith(".zip") || name.endsWith(".jar");
                }
            });
            if (extensions != null) {
                for (int i = 0; i < extensions.length; ++i) {
                    list.add(ext_path + File.separatorChar + extensions[i]);
                }
            }
            else {
                continue;
            }
        }
        StringBuffer buf = new StringBuffer();
        Iterator<Object> e3 = list.iterator();
        while (e3.hasNext()) {
            buf.append((String)e3.next());
            if (!e3.hasNext()) {
                continue;
            }
            buf.append(File.pathSeparatorChar);
        }
        return buf.toString().intern();
    }
    
    public InputStream getInputStream(String name) throws IOException {
        return this.getInputStream(name, ".class");
    }
    
    public InputStream getInputStream(String name, String suffix) throws IOException {
        InputStream is = null;
        try {
            is = this.getClass().getClassLoader().getResourceAsStream(name + suffix);
        }
        catch (Exception ex) {}
        if (is != null) {
            return is;
        }
        return this.getClassFile(name, suffix).getInputStream();
    }
    
    public ClassPath.ClassFile getClassFile(String name, String suffix) throws IOException {
        int i = 0;
        while (i < this.paths.length) {
            ClassPath.ClassFile cf;
            if ((cf = this.paths[i].getClassFile(name, suffix)) != null) {
                return cf;
            }
            ++i;
        }
        throw new IOException("Couldn't find: " + name + suffix);
    }
    
    public ClassPath.ClassFile getClassFile(String name) throws IOException {
        return this.getClassFile(name, ".class");
    }
    
    public byte[] getBytes(String name, String suffix) throws IOException {
        InputStream is = this.getInputStream(name, suffix);
        if (is == null) {
            throw new IOException("Couldn't find: " + name + suffix);
        }
        DataInputStream dis = new DataInputStream(is);
        byte[] bytes = new byte[is.available()];
        dis.readFully(bytes);
        dis.close();
        is.close();
        return bytes;
    }
    
    public byte[] getBytes(String name) throws IOException {
        return this.getBytes(name, ".class");
    }
    
    public String getPath(String name) throws IOException {
        int index = name.lastIndexOf(46);
        String suffix = "";
        if (index > 0) {
            suffix = name.substring(index);
            name = name.substring(0, index);
        }
        return this.getPath(name, suffix);
    }
    
    public String getPath(String name, String suffix) throws IOException {
        return this.getClassFile(name, suffix).getPath();
    }
    
    static {
        SYSTEM_CLASS_PATH = new ClassPath();
    }
    
    private abstract static class PathEntry implements Serializable
    {
        abstract ClassPath.ClassFile getClassFile(String p0, String p1) throws IOException;
    }
    
    private static class Dir extends ClassPath.PathEntry
    {
        private String dir;
        
        Dir(String d) {
            this.dir = d;
        }
        
        @Override
        ClassPath.ClassFile getClassFile(String name, String suffix) throws IOException {
            File file = new File(this.dir + File.separatorChar + name.replace('.', File.separatorChar) + suffix);
            return SecuritySupport.isFileExists(file) ? new ClassPath.ClassFile() {
                @Override
                public InputStream getInputStream() throws IOException {
                    return new FileInputStream(file);
                }
                
                @Override
                public String getPath() {
                    try {
                        return file.getCanonicalPath();
                    }
                    catch (IOException e) {
                        return null;
                    }
                }
                
                @Override
                public long getTime() {
                    return file.lastModified();
                }
                
                @Override
                public long getSize() {
                    return file.length();
                }
                
                @Override
                public String getBase() {
                    return ClassPath.Dir.this.dir;
                }
            } : null;
        }
        
        @Override
        public String toString() {
            return this.dir;
        }
    }
    
    private static class Zip extends ClassPath.PathEntry
    {
        private ZipFile zip;
        
        Zip(ZipFile z) {
            this.zip = z;
        }
        
        @Override
        ClassPath.ClassFile getClassFile(String name, String suffix) throws IOException {
            ZipEntry entry = this.zip.getEntry(name.replace('.', '/') + suffix);
            return (entry != null) ? new ClassPath.ClassFile() {
                @Override
                public InputStream getInputStream() throws IOException {
                    return ClassPath.Zip.this.zip.getInputStream(entry);
                }
                
                @Override
                public String getPath() {
                    return entry.toString();
                }
                
                @Override
                public long getTime() {
                    return entry.getTime();
                }
                
                @Override
                public long getSize() {
                    return entry.getSize();
                }
                
                @Override
                public String getBase() {
                    return ClassPath.Zip.this.zip.getName();
                }
            } : null;
        }
    }
    
    public interface ClassFile
    {
        public abstract InputStream getInputStream() throws IOException;
        
        public abstract String getPath();
        
        public abstract String getBase();
        
        public abstract long getTime();
        
        public abstract long getSize();
    }
}
